# -*- coding: utf-8 -*-
"""media library unit testing"""

from django.conf import settings

from datetime import datetime
from unittest import skipIf

from PIL import Image as PilImage

from django.core.files import File
try:
    from django.urls import reverse
except:
    from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.template.base import TemplateSyntaxError
from django.test.utils import override_settings

from model_mommy import mommy
if 'photologue' in settings.INSTALLED_APPS:
    from photologue.models import Photo, Gallery

from coop_cms.models import ArticleCategory, Document, Image, ImageSize, MediaFilter
from coop_cms.moves import BytesIO, get_response_json
from coop_cms.settings import get_article_class
from coop_cms.tests import BaseArticleTest, BaseTestCase, BeautifulSoup, MediaBaseTestCase


class ImageUploadTest(MediaBaseTestCase):
    """Image upload tests"""
    
    def test_view_form_no_filters(self):
        """view upload image when no filters"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_copyright = soup.select("input#id_copyright")
        self.assertEqual(1, len(id_copyright))
        id_size = soup.select("input#id_size")
        self.assertEqual(1, len(id_size))
        self.assertEqual("hidden", id_size[0]["type"])
        id_filters = soup.select("input#id_filters")
        self.assertEqual(1, len(id_filters))
        self.assertEqual("hidden", id_filters[0]["type"])
        
    def test_view_form_no_sizes(self):
        """view upload image when no sizes"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_size = soup.select("input#id_size")
        self.assertEqual(1, len(id_size))
        self.assertEqual("hidden", id_size[0]["type"])

    def test_view_form_with_filters(self):
        """view upload image when filters"""
        mommy.make(MediaFilter, name="icons")
        mommy.make(MediaFilter, name="big-images")
        
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_filters = soup.select("select#id_filters option")
        self.assertEqual(2, len(id_filters))
    
    def test_view_form_with_sizes(self):
        """view upload image when sizes configured"""
        size1 = mommy.make(ImageSize, name="icons")
        size2 = mommy.make(ImageSize, name="big-images")
        
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)
        id_sizes = soup.select("select#id_size option")
        self.assertEqual(['', str(size1.id), str(size2.id)], [x["value"] for x in id_sizes])
        
    def test_post_form_no_filters(self):
        """upload image when no filters"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': ''
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_size(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        img_size = mommy.make(ImageSize, size="128")
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': '',
            'size': img_size.id,
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, img_size)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_anonymous(self):
        """upload image anonymous user"""
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': ''
        }
        
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(response.status_code, 302)
        next_url = "/accounts/login/?next={0}".format(url)
        self.assertTrue(response['Location'].find(next_url) >= 0)
        
        images = Image.objects.all()
        self.assertEquals(0, images.count())
        
    def test_post_form_not_allowed(self):
        """upload image when user is not allowed"""
        self._log_as_mediamgr()
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': ''
        }
        
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(response.status_code, 403)
        
        images = Image.objects.all()
        self.assertEquals(0, images.count())
        
    def test_post_form_with_filters(self):
        """upload image with filters"""

        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        filter1 = mommy.make(MediaFilter, name="icons")
        mommy.make(MediaFilter, name="big-images")
        filter3 = mommy.make(MediaFilter, name="small-images")
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': [str(f.id) for f in (filter1, filter3)],
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 2)
        self.assertEqual(list(images[0].filters.all().order_by('id')), [filter1, filter3])
        
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_with_filters_no_choice(self):
        """upload image when filters configured and choice is blank"""

        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        mommy.make(MediaFilter, name="icons")
        mommy.make(MediaFilter, name="big-images")
        mommy.make(MediaFilter, name="small-images")
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': [],
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].filters.count(), 0)
        
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())
        
    def test_post_form_with_invalid_size(self):
        """upload image with invalid size"""

        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': [],
            'size': "hhjk",
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.content, b'close_popup_and_media_slide')
        
        images = Image.objects.all()
        self.assertEquals(0, images.count())

    def test_post_form(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': '',
            'filters': '',
            'size': '',
            'copyright': '',
            'alt_text': '',
            'title': '',
        }

        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')

        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, 'unittest1')
        self.assertEqual(images[0].copyright, data['copyright'])
        self.assertEqual(images[0].title, data['title'])
        self.assertEqual(images[0].alt_text, data['alt_text'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())

    def test_post_form_copyright(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': '',
            'size': '',
            'copyright': 'copyright me',
            'title': '',
            'alt_text': '',
        }

        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')

        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].copyright, data['copyright'])
        self.assertEqual(images[0].title, data['title'])
        self.assertEqual(images[0].alt_text, data['alt_text'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())

    def test_post_form_title_and_alt(self):
        """upload image with size"""
        self._log_as_mediamgr(perm=self._permission("add", Image))
        url = reverse('coop_cms_upload_image')
        data = {
            'image': self._get_file("unittest1.png"),
            'descr': 'a test file',
            'filters': '',
            'size': '',
            'copyright': '',
            'title': 'Title',
            'alt_text': 'Alt text',
        }
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        images = Image.objects.all()
        self.assertEquals(1, images.count())
        self.assertEqual(images[0].name, data['descr'])
        self.assertEqual(images[0].copyright, data['copyright'])
        self.assertEqual(images[0].title, data['title'])
        self.assertEqual(images[0].alt_text, data['alt_text'])
        self.assertEqual(images[0].filters.count(), 0)
        self.assertEqual(images[0].size, None)
        file_ = images[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file("unittest1.png").read())


class ImageSizeTest(MediaBaseTestCase):
    """Test media library slide"""
    
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="600")
    def test_image_max_width_no_size(self):
        """get image with no size upscale"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 130)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="600")
    def test_image_max_width_size(self):
        """get image with size"""
        size = mommy.make(ImageSize, size="60", _create_files=True)
        image = mommy.make(Image, size=size, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 60)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="60")
    def test_image_max_width_size_no_scale(self):
        """get image with size downscale"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 60)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="coop_cms.tests.dummy_image_width")
    def test_image_max_width_size_lambda(self):
        """get image from function"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 20)
        
    @override_settings(COOP_CMS_MAX_IMAGE_WIDTH="")
    def test_image_max_width_size_none(self):
        """get image with size setting is empty"""
        image = mommy.make(Image, _create_files=True)
        url = image.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        data = BytesIO(self.get_safe_content(response))
        img = PilImage.open(data)
        self.assertEqual(img.size[0], 130)

    def test_image_no_size(self):
        """get image: no size"""
        image = mommy.make(Image, size=None, _create_files=True)
        url = image.get_absolute_url()
        self.assertEqual(url, image.file.url)

    def test_image_size(self):
        """get image: with size"""
        image_size = mommy.make(ImageSize, size="128x128")
        image = mommy.make(Image, size=image_size, _create_files=True)
        url = image.get_absolute_url()
        self.assertNotEqual(url, image.file.url)

    def test_image_wrong_size(self):
        """get image: wrong size"""
        image_size = mommy.make(ImageSize, size="blabla")
        image = mommy.make(Image, size=image_size, _create_files=True)
        url = image.get_absolute_url()
        self.assertEqual(url, image.file.url)

    def test_image_size_crop(self):
        """get image: cropped"""
        image_size = mommy.make(ImageSize, size="128x128", crop="center")
        image = mommy.make(Image, size=image_size, _create_files=True)
        url = image.get_absolute_url()
        self.assertNotEqual(url, image.file.url)


class MediaLibraryTest(MediaBaseTestCase):
    """Test media library slide"""

    def test_show_images_empty(self):
        """show images empty"""
        self._log_as_mediamgr()
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        
    def test_show_documents_empty(self):
        """show documents empty"""
        self._log_as_mediamgr()
        url = reverse('coop_cms_media_documents')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
            
    def test_show_media_anonymous(self):
        """show images empty anonymous"""
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertNotAllowed(response)
        
    def test_show_media_not_staff(self):
        """show images empty user is not a staff member"""
        self._log_as_mediamgr(is_staff=False)
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(403, response.status_code)
    
    def test_show_images(self):
        """show images"""
        self._log_as_mediamgr()
        mommy.make(Image, _quantity=2, _create_files=True)
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))
        
    def test_show_images_pagination(self):
        """show images with pagination"""
        self._log_as_mediamgr()
        mommy.make(Image, _quantity=16, _create_files=True)
        url = reverse('coop_cms_media_images')
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(12, len(nodes))
    
    def test_show_images_page_2(self):
        """show images page 2"""
        self._log_as_mediamgr()
        mommy.make(Image, _quantity=16, _create_files=True)
        url = reverse('coop_cms_media_images')+"?page=2"
        response = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(200, response.status_code)
        data = get_response_json(response)
        soup = BeautifulSoup(data['html'])
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(4, len(nodes))
        
    def test_show_images_media_filter(self):
        """show images with media filter"""
        self._log_as_mediamgr()
        media_filter = mommy.make(MediaFilter)
        images = []
        for i in range(16):
            images.append(mommy.make(Image, created=datetime(2014, 1, 1, 12, i), _create_files=True))
        images.reverse()
        
        images[5].filters.add(media_filter)
        images[15].filters.add(media_filter)
        url = reverse('coop_cms_media_images')+"?page=1&media_filter={0}".format(media_filter.id)
        response = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(200, response.status_code)
        data = get_response_json(response)
        soup = BeautifulSoup(data['html'])
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))
        expected = [x.file.url for x in (images[5], images[15])]
        actual = [node["rel"] for node in nodes]
        self.assertEqual(expected, actual)
    
    def test_show_images_media_filter_all(self):
        """show images empty with all media filter"""
        self._log_as_mediamgr()
        media_filter = mommy.make(MediaFilter)
        
        images = []
        for i in range(16):
            images.append(mommy.make(Image, created=datetime(2014, 1, 1, 12, i), _create_files=True))
        images.reverse()
        
        images[5].filters.add(media_filter)
        images[15].filters.add(media_filter)
        url = reverse('coop_cms_media_images')+"?page=1&media_filter={0}".format(0)
        response = self.client.get(url, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(200, response.status_code)
        data = get_response_json(response)
        soup = BeautifulSoup(data['html'])
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(12, len(nodes))
        expected = [x.file.url for x in images[:12]]
        actual = [node["rel"] for node in nodes]
        self.assertEqual(expected, actual)
    
    
class UploadDocTest(MediaBaseTestCase):
    """Download document"""

    def test_upload_public_doc(self):
        """upload public"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        data = {
            'file': self._get_file(),
            'is_private': False,
            'name': 'a test file',
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        public_docs = Document.objects.filter(is_private=False)
        self.assertEquals(1, public_docs.count())
        self.assertEqual(public_docs[0].name, data['name'])
        self.assertEqual(public_docs[0].category, None)
        file_ = public_docs[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file().read())
        
    def test_upload_public_doc_category(self):
        """upload public category"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        cat = mommy.make(ArticleCategory, name="my cat")
        data = {
            'file': self._get_file(),
            'is_private': False,
            'name': 'a test file',
            'category': cat.id,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        public_docs = Document.objects.filter(is_private=False)
        self.assertEquals(1, public_docs.count())
        self.assertEqual(public_docs[0].name, data['name'])
        self.assertEqual(public_docs[0].category, cat)
        file_ = public_docs[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file().read())
        
    def test_upload_doc_missing_fields(self):
        """upload public: missing fields"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        data = {
            'is_private': False,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotEqual(response.content, b'close_popup_and_media_slide')
        self.assertEquals(0, Document.objects.all().count())

    def test_upload_doc_anonymous_user(self):
        """upload: anonymous user"""

        data = {
            'file': self._get_file(),
            'is_private': False,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data)
        self.assertNotAllowed(response)

    def test_upload_not_allowed(self):
        """upload: not allowed"""

        self._log_as_mediamgr()
        data = {
            'file': self._get_file(),
            'is_private': False,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 403)
        
    def test_upload_private_doc(self):
        """upload private doc"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        data = {
            'file': self._get_file(),
            'is_private': True,
        }
        response = self.client.post(reverse('coop_cms_upload_doc'), data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'close_popup_and_media_slide')
        private_docs = Document.objects.filter(is_private=True)
        self.assertEquals(1, private_docs.count())
        #TODO : on drone.io filename is unittest1_SomeRandom. Why?
        #self.assertNotEqual(private_docs[0].name, 'unittest1')
        self.assertNotEqual(private_docs[0].name, '')
        self.assertEqual(private_docs[0].category, None)
        file_ = private_docs[0].file
        file_.open('rb')
        self.assertEqual(file_.read(), self._get_file().read())


class DocsInMediaLibTest(MediaBaseTestCase):
    """view documents in media lib"""

    def test_view_docs(self):
        """view documents"""
        self._log_as_mediamgr(perm=self._permission("add", Document))
        file1 = File(self._get_file())
        doc1 = mommy.make(Document, is_private=True, file=file1)
        file2 = File(self._get_file())
        doc2 = mommy.make(Document, is_private=False, file=file2)
        
        response = self.client.get(reverse('coop_cms_media_documents'))
        self.assertEqual(response.status_code, 200)
        
        self.assertContains(response, reverse('coop_cms_download_doc', args=[doc1.id]))
        self.assertNotContains(response, doc1.file.url)
        self.assertNotContains(response, reverse('coop_cms_download_doc', args=[doc2.id]))
        self.assertContains(response, doc2.file.url)
        
    def test_view_docs_anonymous(self):
        """view docs anonymous"""
        url = reverse('coop_cms_media_documents')
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        redirect_url = response.redirect_chain[-1][0]
        login_url = reverse('login')
        self.assertTrue(redirect_url.find(login_url) >= 0)
        
    def test_view_docs_not_allowed(self):
        """view docs not allowed"""
        self._log_as_mediamgr(is_staff=False)
        response = self.client.get(reverse('coop_cms_media_documents'), follow=True)
        self.assertEqual(response.status_code, 403)


@skipIf('photologue' not in settings.INSTALLED_APPS, "photologue is not installed")
class PhotologueInMediaLibTest(MediaBaseTestCase):
    """view documents in media lib"""

    def test_view_photos_empty(self):
        """view medialib when no photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(0, len(nodes))

        nodes = soup.select(".media-filters a")
        self.assertEqual(0, len(nodes))

    def test_view_photos_not_empty(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        mommy.make(Photo)
        mommy.make(Photo)

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))

        nodes = soup.select(".media-filters a")
        self.assertEqual(0, len(nodes))

    def test_view_anonymous(self):
        """view docs anonymous"""
        response = self.client.get(reverse('coop_cms_media_photologue'), follow=True)
        self.assertEqual(response.status_code, 200)
        redirect_url = response.redirect_chain[-1][0]
        login_url = reverse('login')
        self.assertTrue(redirect_url.find(login_url) >= 0)

    def test_view_not_allowed(self):
        """view docs not allowed"""
        self._log_as_mediamgr(is_staff=False)
        response = self.client.get(reverse('coop_cms_media_photologue'), follow=True)
        self.assertEqual(response.status_code, 403)

    def test_view_photos_gallery(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        gallery = mommy.make(Gallery)

        photo1 = mommy.make(Photo)
        photo1.galleries.add(gallery)
        photo1.save()

        mommy.make(Photo)

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))

        nodes = soup.select(".media-filters a")
        #All + gallery
        self.assertEqual(2, len(nodes))

    def test_view_photos_gallery_several(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        gallery = mommy.make(Gallery)
        mommy.make(Gallery)
        mommy.make(Gallery)

        photo1 = mommy.make(Photo)
        photo1.galleries.add(gallery)
        photo1.save()

        mommy.make(Photo)

        response = self.client.get(reverse('coop_cms_media_photologue'))
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(2, len(nodes))

        nodes = soup.select(".media-filters a")
        #All + galleries
        self.assertEqual(4, len(nodes))

    def test_view_photos_only_gallery(self):
        """view photos"""
        self._log_as_mediamgr(perm=self._permission("add", Photo))
        gallery1 = mommy.make(Gallery)
        gallery2 = mommy.make(Gallery)

        photo1 = mommy.make(Photo)
        photo1.galleries.add(gallery1)
        photo1.save()

        photo2 = mommy.make(Photo)
        photo2.galleries.add(gallery2)
        photo2.save()

        mommy.make(Photo)

        url = reverse('coop_cms_media_photologue')+"?gallery_filter={0}".format(gallery1.id)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        nodes = soup.select(".library-thumbnail")
        self.assertEqual(1, len(nodes))
        self.assertEqual(photo1.title, nodes[0]["title"])

        nodes = soup.select(".media-filters a")
        self.assertEqual(3, len(nodes))


class DownloadDocTest(MediaBaseTestCase):
    """Download document"""

    def test_download_public(self):
        """download public doc"""
        #create a public doc
        file_ = File(self._get_file())
        doc = mommy.make(Document, is_private=False, file=file_)
        
        #check the url
        private_url = reverse('coop_cms_download_doc', args=[doc.id])
        self.assertNotEqual(doc.get_download_url(), private_url)
        
        #login and download
        self._log_as_mediamgr()

        response = self.client.get(doc.get_download_url())
        self.assertEqual(response.status_code, 200)

        content = self.get_safe_content(response)
        self.assertEqual(content, self._get_file().read())
        
        #logout and download
        self.client.logout()
        response = self.client.get(doc.get_download_url())
        self.assertEqual(response.status_code, 200)
        content = self.get_safe_content(response)
        self.assertEqual(content, self._get_file().read())
        
    @skipIf('sanza.Profile' in settings.INSTALLED_APPS, "sanza.Profile installed")
    def test_download_private(self):
        """download private doc"""
            
        #create a public doc
        file_ = File(self._get_file())
        cat = mommy.make(ArticleCategory, name="private-doc")
        doc = mommy.make(Document, is_private=True, file=file_, category=cat)

        #check the url
        private_url = reverse('coop_cms_download_doc', args=[doc.id])
        self.assertEqual(doc.get_download_url(), private_url)
        
        #login and download
        self._log_as_mediamgr()
        response = self.client.get(doc.get_download_url())
        self.assertEqual(response.status_code, 200)
        self.assertEquals(response['Content-Disposition'], "attachment; filename=unittest1.txt")
        self.assertEquals(response['Content-Type'], "text/plain")
        # TODO: This change I/O Exception in UnitTest
        # self.assertEqual(response.content, self._get_file().read())
        
        # logout and download
        self.client.logout()
        response = self.client.get(doc.get_download_url())
        self.assertNotAllowed(response)
        

class ImageListTemplateTagTest(BaseTestCase):
    """Test image List template tag"""

    def test_non_existing_filter(self):
        """test not existing ist"""
        tpl = Template('{% load coop_utils %}{% coop_image_list "test" as image_list %}{{image_list|length}}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "0")

    def test_empty_filter(self):
        """test empty filter"""
        mommy.make(MediaFilter, name="abcd")
        tpl = Template('{% load coop_utils %}{% coop_image_list "abcd" as image_list %}{{image_list|length}}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "0")

    def test_filter_with_images(self):
        """test filter with image"""
        filter_ = mommy.make(MediaFilter, name="abcd")
        mommy.make(Image, filters=[filter_], _create_files=True)
        tpl = Template('{% load coop_utils %}{% coop_image_list "abcd" as image_list %}{{image_list|length}}')
        html = tpl.render(Context({}))
        self.assertEqual(html, "1")

    def test_filter_with_images_var_name(self):
        """test filter: name from variable"""
        filter_ = mommy.make(MediaFilter, name="abcd")
        mommy.make(Image, filters=[filter_], _create_files=True)
        tpl = Template('{% load coop_utils %}{% coop_image_list filter_name as image_list %}{{image_list|length}}')
        html = tpl.render(Context({"filter_name": filter_.name}))
        self.assertEqual(html, "1")

    def test_filter_as_missing(self):
        """test filter: as is missing"""
        filter_ = mommy.make(MediaFilter, name="abcd")
        mommy.make(Image, filters=[filter_], _create_files=True)
        try:
            Template('{% load coop_utils %}{% coop_image_list "abcd" image_list %}{{image_list|length}}')
        except TemplateSyntaxError as msg:
            self.assertEqual(
                "coop_image_list: usage --> {% coop_image_list 'filter_name' as var_name %}",
                '{0}'.format(msg)
            )
        else:
            self.assertEqual("", "No exception")


class ArticleLogoTest(BaseArticleTest):
    """test article logo"""

    def _get_image(self, file_name='unittest1.png'):
        """get image"""
        return self._get_file(file_name)

    def setUp(self):
        """before each test"""
        super(ArticleLogoTest, self).setUp()
        self._default_article_templates = settings.COOP_CMS_ARTICLE_TEMPLATES
        settings.COOP_CMS_ARTICLE_TEMPLATES = (
            ('test/article_with_logo_size.html', 'Article with logo size'),
            ('test/article_with_logo_size_and_crop.html', 'Article with logo size and crop'),
            ('test/article_no_logo_size.html', 'Article no logo size and crop'),
        )
        self._default_logo_size = getattr(settings, 'COOP_CMS_ARTICLE_LOGO_SIZE', None)
        self._default_logo_crop = getattr(settings, 'COOP_CMS_ARTICLE_LOGO_CROP', None)

    def tearDown(self):
        """after each test"""
        super(ArticleLogoTest, self).tearDown()
        #restore
        settings.COOP_CMS_ARTICLE_TEMPLATES = self._default_article_templates
        settings.COOP_CMS_ARTICLE_LOGO_SIZE = self._default_logo_size
        settings.COOP_CMS_ARTICLE_LOGO_CROP = self._default_logo_crop

    def test_view_article_no_image(self, template_index=0, image=False):
        """article no image"""
        article_class = get_article_class()
        article = mommy.make(
            article_class,
            title="This is my article",
            content="<p>This is my <b>content</b></p>",
            template=settings.COOP_CMS_ARTICLE_TEMPLATES[template_index][0]
        )
        if image:
            article.logo = File(self._get_image())
            article.save()

        response = self.client.get(article.get_absolute_url())
        self.assertEqual(response.status_code, 200)

        self.assertContains(response, article.title)
        self.assertContains(response, article.content)

    def test_view_article_no_image_crop(self):
        """test article no image crop"""
        self.test_view_article_no_image(1)

    def test_view_article_image_no_crop(self):
        """test article no image no crop"""
        self.test_view_article_no_image(0, True)

    def test_view_article_image_and_crop(self):
        """test article image and crop"""
        self.test_view_article_no_image(1, True)

    def test_edit_article_no_image(self, template_index=0, image=False, post_image=False):
        """test edit article no image"""
        article_class = get_article_class()
        if image:
            article = mommy.make(
                article_class,
                title="This is my article",
                content="<p>This is my <b>content</b></p>",
                slug="",
                template=settings.COOP_CMS_ARTICLE_TEMPLATES[template_index][0],
                logo=File(self._get_image())
            )
        else:
            article = mommy.make(
                article_class,
                title="This is my article",
                content="<p>This is my <b>content</b></p>",
                slug="",
                template=settings.COOP_CMS_ARTICLE_TEMPLATES[template_index][0]
            )

        self._log_as_editor()

        response = self.client.post(article.get_edit_url(), follow=True)
        self.assertEqual(response.status_code, 200)

        data = {
            'title': 'Title of the article',
            'content': 'The content',
        }
        if post_image:
            data['logo'] = self._get_image('unittest2.png')
        response = self.client.post(article.get_edit_url(), data=data, follow=True)
        self.assertEqual(response.status_code, 200)

        article = article_class.objects.get(id=article.id)

        self.assertEqual(data['title'], article.title)
        self.assertEqual(data['content'], article.content)

        self.assertContains(response, article.title)
        self.assertContains(response, article.content)

    def test_edit_article_no_image_crop(self):
        """test edit article no image and crop"""
        self.test_edit_article_no_image(1)

    def test_edit_article_image_no_post_no_crop(self):
        """test edit article image no crop"""
        self.test_edit_article_no_image(0, True, False)

    def test_edit_article_image_no_post_crop(self):
        """test edit article image no crop"""
        self.test_edit_article_no_image(1, True, False)

    def test_edit_article_image_post_no_crop(self):
        """test edit article image no crop"""
        self.test_edit_article_no_image(0, True, True)

    def test_edit_article_image_post_crop(self):
        """test edit article image crop"""
        self.test_edit_article_no_image(1, True, True)

    def test_view_article_no_image_template1(self):
        """test view article no image template 1"""
        self.test_view_article_no_image(2, False)

    def test_view_article_no_image_template2(self):
        """test view article no image template 2"""
        self.test_view_article_no_image(2, True)

    def test_view_article_no_image_template3(self):
        """test view article no image template 3"""
        settings.COOP_CMS_ARTICLE_LOGO_SIZE = "x100"
        settings.COOP_CMS_ARTICLE_LOGO_CROP = "top"
        self.test_view_article_no_image(2, True)

    def test_edit_article_no_image_template1(self):
        """test edit article no image template 1"""
        self.test_edit_article_no_image(2, False, False)

    def test_edit_article_no_image_template2(self):
        """test edit article no image template 2"""
        self.test_edit_article_no_image(2, True, False)

    def test_edit_article_no_image_template3(self):
        """test edit article no image template 3"""
        self.test_edit_article_no_image(2, True, True)

    def test_edit_article_no_image_template4(self):
        """test edit article no image template 4"""
        settings.COOP_CMS_ARTICLE_LOGO_SIZE = "x100"
        settings.COOP_CMS_ARTICLE_LOGO_CROP = "top"
        self.test_edit_article_no_image(2, True, True)



# -*- coding: utf-8 -*-
"""create newsletter item"""

from django.core.management.base import BaseCommand

from coop_cms.models import create_newsletter_item
from coop_cms.settings import get_newsletter_item_classes


class Command(BaseCommand):
    """force the creation of every newsletter items"""
    help = u"force the creation of every newsletter items"
    use_argparse = False

    def handle(self, *args, **options):
        """command"""
        # look for emailing to be sent
        verbose = options.get('verbosity', 1)
        
        for klass in get_newsletter_item_classes():
            for instance in klass.objects.all():
                item, status = create_newsletter_item(instance)
                if item and status and verbose:
                    print('create item for', klass.__name__, '-', instance.id, ':', instance)
                else:
                    if status and verbose:
                        print('delete item for', klass.__name__, '-', instance.id, ':', instance)
        if verbose:
            print("done")



# -*- coding: utf-8 -*-
"""unit test i18n support"""

from unittest import skipIf

from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
try:
    from django.urls import reverse
except:
    from django.core.urlresolvers import reverse
from django.utils.translation import activate, get_language
from django.test.utils import override_settings

from coop_cms.models import BaseArticle, InvalidArticleError
from coop_cms.settings import has_localized_urls, is_localized, is_multilang, multilang_mode, get_article_class
from coop_cms.tests import BaseTestCase, BeautifulSoup
from coop_cms.utils import redirect_to_language, strip_locale_path, get_url_in_language, make_locale_path


def language_fallbacks():
    """return fallback"""
    return tuple([lang_code for (lang_code, lang_name) in settings.LANGUAGES])


@skipIf(not has_localized_urls(), "Urls are not localized")
class LocalePathTest(BaseTestCase):
    """test that url is parsed correctly when using locale prefix"""

    def test_get_locale_article(self):
        """it should return lang and locale-independent-path"""
        result = strip_locale_path('/en/home/')
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], 'en')
        self.assertEqual(result[1], '/home/')

    def test_get_locale_article_no_trailing(self):
        """it should return lang and locale-independent-path"""
        result = strip_locale_path('/en/home')
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], 'en')
        self.assertEqual(result[1], '/home')

    def test_get_locale_article_no_prefix(self):
        """it should returns empty locale and path"""
        result = strip_locale_path('/home/')
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], '')
        self.assertEqual(result[1], '/home/')

    def test_get_locale_article_no_prefix_no_trailing(self):
        """it should returns empty locale and path"""
        result = strip_locale_path('/home')
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], '')
        self.assertEqual(result[1], '/home')

    @skipIf(not is_multilang(), "not multi lang")
    def test_get_url_in_language(self):
        """it should return url in another language"""

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]

        activate(origin_lang)
        path = '/this-is-test/'
        origin_url = make_locale_path(path, origin_lang)

        trans_url = get_url_in_language(origin_url, trans_lang)
        self.assertEqual(trans_url, make_locale_path(path, trans_lang))


class UrlLocalizationTest(BaseTestCase):
    """localize url"""
    
    def setUp(self):
        self.user = None
        activate(settings.LANGUAGES[0][0])
    
    def tearDown(self):
        activate(settings.LANGUAGES[0][0])
    
    def _log_as_editor(self):
        """Log as editor"""
        self.user = user = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
        
        content_type = ContentType.objects.get_for_model(get_article_class())
        
        perm = 'change_{0}'.format(content_type.model)
        can_edit_article = Permission.objects.get(content_type=content_type, codename=perm)
        user.user_permissions.add(can_edit_article)
        
        perm = 'add_{0}'.format(content_type.model)
        can_add_article = Permission.objects.get(content_type=content_type, codename=perm)
        user.user_permissions.add(can_add_article)
        
        user.is_active = True
        user.save()
        return self.client.login(username='toto', password='toto')
    
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_get_locale_article(self):
        """get article with locale slug"""
        original_text = '*!-+' * 10
        translated_text = ':%@/' * 9
        
        art1 = get_article_class().objects.create(title="Home", content=original_text)
        
        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        
        setattr(art1, 'title_' + trans_lang, 'Accueil')
        setattr(art1, 'content_' + trans_lang, translated_text)
        art1.save()
        
        response = self.client.get('/{0}/home/'.format(origin_lang), follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, original_text)
        
        response = self.client.get('/{0}/accueil/'.format(trans_lang), follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, translated_text)

    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_change_lang(self):
        """change language"""

        original_text = '*!-+' * 10
        translated_text = ':%@/' * 9
        
        art1 = get_article_class().objects.create(title="Home", content=original_text)
        
        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]

        activate(origin_lang)

        setattr(art1, 'title_' + trans_lang, 'Accueil')
        setattr(art1, 'content_' + trans_lang, translated_text)
        
        art1.save()
        
        origin_url = '/{0}/home/'.format(origin_lang)
        response = self.client.get(origin_url, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, original_text)
        
        data = {'language': trans_lang}
        response = self.client.post(
            reverse('coop_cms_change_language') + '?next={0}'.format(origin_url),
            data=data,
            follow=True
        )
        self.assertEqual(200, response.status_code)
        self.assertContains(response, translated_text)
        
        response = self.client.get('/{0}/accueil/'.format(trans_lang), follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, translated_text)
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_change_lang_next_url_after(self):
        """change language and redirect to url"""
        
        original_text = '*!-+' * 10
        translated_text = ':%@/' * 9
        
        art1 = get_article_class().objects.create(title="Home", content=original_text)
        
        art2 = get_article_class().objects.create(title="Next", content="****NEXT****")
        
        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        
        setattr(art1, 'title_' + trans_lang, 'Accueil')
        setattr(art1, 'content_' + trans_lang, translated_text)
        
        art1.save()
        
        origin_url = '/{0}/home/'.format(origin_lang)
        response = self.client.get(origin_url, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, original_text)
        
        data = {'language': trans_lang, 'next_url_after_change_lang': art2.get_absolute_url()}
        response = self.client.post(
            reverse('coop_cms_change_language'),
            data=data,
            follow=True
        )
        self.assertEqual(200, response.status_code)
        self.assertContains(response, art2.content)
            
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_change_lang_no_trans(self):
        """change language and no translation"""

        original_text = '*!-+' * 10
        
        get_article_class().objects.create(title="Home", content=original_text)
        
        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        
        origin_url = '/{0}/home/'.format(origin_lang)
        response = self.client.get(origin_url, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, original_text)
        
        data = {'language': trans_lang}
        response = self.client.post(
            reverse('coop_cms_change_language') + '?next={0}'.format(origin_url),
            data=data,
            follow=True
        )
        self.assertEqual(200, response.status_code)
        self.assertContains(response, original_text)
        
        response = self.client.get('/{0}/home/'.format(trans_lang), follow=True)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, original_text)
            
    def test_keep_slug(self):
        """test slug are not modified when changing title"""
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", content="aa")
        original_slug = art1.slug
        art1.title = "Title changed"
        art1.save()
        art1 = article_class.objects.get(id=art1.id)
        self.assertEqual(original_slug, art1.slug)
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_keep_localized_slug(self):
        """test translation of slug are not modified when changing title"""
        
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", content="aa")
        trans_lang = settings.LANGUAGES[1][0]
        setattr(art1, 'title_' + trans_lang, 'Accueil')
        art1.save()
        
        original_slug = art1.slug
        original_trans_slug = getattr(art1, 'slug_' + trans_lang, '**dummy**')
        
        art1.title = "Title changed"
        setattr(art1, 'title_' + trans_lang, 'Titre change')
        
        art1.save()
        art1 = article_class.objects.get(id=art1.id)
        
        self.assertEqual(original_slug, art1.slug)
        self.assertEqual(original_trans_slug, getattr(art1, 'slug_' + trans_lang))
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localized_slug_already_existing(self):
        """test localized slug already exists"""
        
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", content="aa")
        art2 = article_class.objects.create(title="Rome", content="aa")

        trans_lang = settings.LANGUAGES[1][0]
        setattr(art1, 'title_' + trans_lang, art2.title)
        art1.save()
        
        art2.save()
        
        setattr(art2, 'title_' + trans_lang, art2.title)
        art2.save()
        
        self.assertNotEqual(getattr(art2, 'slug_' + trans_lang), getattr(art1, 'slug_' + trans_lang))
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localized_slug_already_existing2(self):
        """test localized slug already exists 2 """
        
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", content="aa")
        art2 = article_class.objects.create(title="Rome", content="aa")

        trans_lang = settings.LANGUAGES[1][0]
        setattr(art1, 'title_' + trans_lang, art2.title)
        art1.save()
        
        setattr(art2, 'title_' + trans_lang, art2.title)
        art2.save()
        
        self.assertNotEqual(getattr(art2, 'slug_' + trans_lang), getattr(art1, 'slug_' + trans_lang))
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localized_slug_already_existing3(self):
        """test localized slug already exists 3 """
        self._log_as_editor()
        article_class = get_article_class()

        art1 = article_class.objects.create(title="Home", content="aa")
        art2 = article_class.objects.create(title="Rome", content="aa", template='test/article.html')

        trans_lang = settings.LANGUAGES[1][0]
        setattr(art1, 'title_' + trans_lang, art2.title)
        art1.save()
        
        # CHANGE LANGUAGE
        activate(trans_lang)
        
        url = art2.get_edit_url()
        
        data = {
            'title': art2.title,
            'content': 'translation', 
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        art2_updated = article_class.objects.get(id=art2.id)
        
        self.assertEqual(getattr(art2_updated, 'title_' + trans_lang), art2.title)
        
        self.assertNotEqual(getattr(art2_updated, 'slug_' + trans_lang), getattr(art1, 'slug_' + trans_lang))
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localize_existing_article1(self):
        """test localized existing article 1 """

        self._log_as_editor()
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", template='test/article.html')

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        
        # CHANGE LANGUAGE
        activate(trans_lang)
        
        url = art1.get_edit_url()
        
        data = {
            'title': "Home",
            'content': 'translation', 
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        art1_updated = article_class.objects.get(id=art1.id)
        
        self.assertEqual(getattr(art1_updated, 'title_' + trans_lang), art1.title)
        self.assertEqual(getattr(art1_updated, 'slug_' + trans_lang), getattr(art1, 'slug_' + origin_lang))
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localize_existing_article2(self):
        """test localized existing article 2 """
        self._log_as_editor()
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Accueil", template='test/article.html')

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]
        
        # CHANGE LANGUAGE
        activate(trans_lang)
        
        url = art1.get_edit_url()
        
        data = {
            'title': "Home",
            'content': 'translation', 
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        art1_updated = article_class.objects.get(id=art1.id)
        self.assertEqual(getattr(art1_updated, 'title_' + origin_lang), art1.title)
        self.assertEqual(getattr(art1_updated, 'title_' + trans_lang), data["title"])
        self.assertEqual(getattr(art1_updated, 'slug_' + trans_lang), "home")
        self.assertEqual(getattr(art1_updated, 'slug_' + origin_lang), "accueil")
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localized_slug_already_existing4(self):
        """test localized slug and existing article """
        self._log_as_editor()
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", content="aa")
        art2 = article_class.objects.create(title="Rome", content="aa", template='test/article.html')

        trans_lang = settings.LANGUAGES[1][0]
        
        self.assertEqual(None, getattr(art2, 'slug_' + trans_lang))
        
        # CHANGE LANGUAGE
        activate(trans_lang)
        
        url = art2.get_edit_url()
        
        data = {
            'title': art1.title,
            'content': 'translation', 
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        art2_updated = article_class.objects.get(id=art2.id)
        
        self.assertEqual(getattr(art2_updated, 'title_' + trans_lang), art1.title)
        
        self.assertNotEqual(getattr(art2_updated, 'slug_' + trans_lang), art1.slug)
        
    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_localized_slug_already_existing5(self):
        """test localized slug existing and existing article """

        self._log_as_editor()
        article_class = get_article_class()
        art1 = article_class.objects.create(title="Home", content="aa")
        art2 = article_class.objects.create(title="Rome", content="aa", template='test/article.html')

        trans_lang = settings.LANGUAGES[1][0]
        
        self.assertEqual(None, getattr(art2, 'slug_' + trans_lang))
        
        setattr(art2, 'title_' + trans_lang, art1.title)
        art2.save()
        self.assertNotEqual(art1.slug, getattr(art2, 'slug_' + trans_lang))
        
        # CHANGE LANGUAGE
        activate(trans_lang)
        
        url = art2.get_edit_url()
        
        data = {
            'title': art1.title,
            'content': 'translation', 
        }
        
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(response.status_code, 200)
        
        art2_updated = article_class.objects.get(id=art2.id)
        
        self.assertEqual(getattr(art2_updated, 'title_' + trans_lang), art1.title)
        
        self.assertNotEqual(getattr(art2_updated, 'slug_' + trans_lang), art1.slug)
            
    def test_no_title(self):
        """test create article without title"""
        article_class = get_article_class()
        
        try:
            article_class.objects.create(title="", content="a!*%:" * 10, publication=BaseArticle.PUBLISHED)
        except InvalidArticleError:
            # OK
            return

        # Force to fail
        self.assertFalse(True)

    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    @override_settings(MODELTRANSLATION_FALLBACK_LANGUAGES=language_fallbacks())
    def test_create_article_in_additional_lang_fallback(self):
        """test create article into an other language than the default"""

        article_class = get_article_class()
        
        default_lang = settings.LANGUAGES[0][0]
        other_lang = settings.LANGUAGES[1][0]
        
        activate(other_lang)
        
        art1 = article_class.objects.create(title="abcd", content="a!*%:" * 10, publication=BaseArticle.PUBLISHED)
        
        response = self.client.get(art1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, art1.content)
        
        activate(default_lang)
        
        response = self.client.get(art1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, art1.content)

    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_create_article_in_additional_lang_no_fallback(self):
        """test create article into an other language than the default"""

        article_class = get_article_class()

        default_lang = settings.LANGUAGES[0][0]
        other_lang = settings.LANGUAGES[1][0]

        activate(other_lang)

        art1 = article_class.objects.create(title="abcd", content="a!*%:" * 10, publication=BaseArticle.PUBLISHED)

        response = self.client.get(art1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertContains(response, art1.content)

        activate(default_lang)

        response = self.client.get(art1.get_absolute_url())
        self.assertEqual(404, response.status_code)

    @skipIf(not has_localized_urls() or multilang_mode() < 3, "not localized")
    def test_create_article_in_third_lang(self):
        """test create article into an other language than the default"""

        article_class = get_article_class()

        default_lang = settings.LANGUAGES[0][0]
        other_lang = settings.LANGUAGES[1][0]
        third_lang = settings.LANGUAGES[2][0]

        activate(third_lang)
        content = "a!*%:"*10

        art1 = article_class.objects.create(title="abcd", content=content, publication=BaseArticle.PUBLISHED)
        self.assertEqual(art1.slug, 'abcd')
        third_lang_url = art1.get_absolute_url()
        response = self.client.get(third_lang_url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, content)

        activate(other_lang)
        art1.title = 'efgh'
        art1.save()

        self.assertEqual(art1.slug, 'efgh')

        response = self.client.get(art1.get_absolute_url())
        self.assertEqual(200, response.status_code)
        self.assertNotContains(response, content)

        response = self.client.get(third_lang_url.replace('/' + third_lang + '/', '/' + other_lang + '/', 1))
        self.assertEqual(404, response.status_code)

    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_redirect_to_language(self):
        """check redirect_to_language utility"""
        article_class = get_article_class()

        other_lang = settings.LANGUAGES[1][0]

        art1 = article_class.objects.create(title="abcd", publication=BaseArticle.PUBLISHED)

        response = redirect_to_language(art1.get_absolute_url(), other_lang)

        self.assertTrue(response.url.find("/" + other_lang + "/") == 0)
        self.assertEqual(get_language(), other_lang)

    @skipIf(not has_localized_urls() or not is_multilang(), "not localized")
    def test_redirect_to_invalid_language(self):
        """check redirect_to_language uitiliy raise error if ImproperlyConfigured"""

        article_class = get_article_class()

        art1 = article_class.objects.create(title="abcd", publication=BaseArticle.PUBLISHED)

        self.assertRaises(ImproperlyConfigured, redirect_to_language, art1.get_absolute_url(), "zz")


class SwitchLanguageTest(BaseTestCase):
    """change the language from coop-bar popup"""

    def setUp(self):
        self.user = None
        activate(settings.LANGUAGES[0][0])

    def tearDown(self):
        activate(settings.LANGUAGES[0][0])

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_get_language_switcher(self):
        """get article with locale slug"""

        original_text = '*!-+' * 10
        translated_text = ':%@/' * 9

        art1 = get_article_class().objects.create(title="Home", content=original_text)

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]

        setattr(art1, 'title_' + trans_lang, 'Accueil')
        setattr(art1, 'content_' + trans_lang, translated_text)
        art1.save()

        url = reverse('coop_cms_switch_language_popup')

        response = self.client.get(url)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select('select#id_language option')), len(settings.LANGUAGES))

    @skipIf(not is_localized() or not is_multilang(), "not localized")
    def test_change_lang(self):
        """change language"""

        original_text = '*!-+' * 10
        translated_text = ':%@/' * 9

        art1 = get_article_class().objects.create(title="Home", content=original_text)

        origin_lang = settings.LANGUAGES[0][0]
        trans_lang = settings.LANGUAGES[1][0]

        activate(origin_lang)

        setattr(art1, 'title_' + trans_lang, 'Accueil')
        setattr(art1, 'content_' + trans_lang, translated_text)

        art1.save()

        data = {'language': trans_lang}
        response = self.client.post(
            reverse('coop_cms_switch_language_popup'),
            data=data,
            follow=True
        )
        self.assertEqual(200, response.status_code)
        self.assertEqual(trans_lang, get_language())



# -*- coding: utf-8 -*-
"""links: object who redirects to another url"""

from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages.api import success as success_message
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils.translation import ugettext as _

from colorbox.decorators import popup_redirect

from coop_cms.forms.content import NewLinkForm
from coop_cms import models
from coop_cms.moves import reverse


@login_required(login_url=reverse_lazy('login'))
@popup_redirect
def new_link(request):
    """new link"""
    content_type = ContentType.objects.get_for_model(models.Link)
    perm = '{0}.add_{1}'.format(content_type.app_label, content_type.model)

    if not request.user.has_perm(perm):
        raise PermissionDenied

    if request.method == "POST":
        form = NewLinkForm(request.POST)
        if form.is_valid():
            form.save()
            form.save_m2m()
            homepage_url = reverse('coop_cms_homepage')
            next_url = request.META.get('HTTP_REFERER', homepage_url)
            success_message(request, _('The link has been created properly'))
            return HttpResponseRedirect(next_url)
    else:
        form = NewLinkForm()

    context = {
        'form': form,
    }

    return render(
        request,
        'coop_cms/popup_new_link.html',
        context
    )
